#!/usr/bin/bash

BASE_DIR=$(realpath "$0")
BASE_DIR=$(dirname "$BASE_DIR")
export BASE_DIR
export PATH="${BASE_DIR}:$PATH"

USAGE_S='' DOMAIN_NAME='' PC_NAME='' ADMIN_NAME=''  WINBIND=''  DOMAIN_TYPE='WIN' ADMIN_PASS='' DC1='' DC2='' KRB5_DC1='' KRB5_DC2='' FQN=''

while getopts "d:n:u:whvIq" OPTION; do
  case $OPTION in
    'd') DOMAIN_NAME="$OPTARG" ;;
    'n') PC_NAME="$OPTARG" ;;
    'u') ADMIN_NAME="$OPTARG" ;;
    'w') WINBIND='Y' ;;
    'q') FQN='-q' ;;
    'I') DOMAIN_TYPE='IPA' ;;
    'v') echo "Version: $SCRIPT_VERSION"
          exit ;;
    'h') USAGE_S="Y" ;;
    *) USAGE_S="Y" ;;
  esac
done

shift $((OPTIND-1))
OPTIND=1

ADMIN_PASS=$(</dev/stdin)
. "${BASE_DIR}/join-to-domain-function.sh" || exit 1

LogMess -S -m "Script started from GUI. Version: $SCRIPT_VERSION"
LogMess -m "Time: $CURR_TIME"

[[ -n "$USAGE_S" ]] && { Usage; exit 0; }

ADMIN_NAME=${ADMIN_NAME%%@*}

REALM_CHK=$(realm list)
if [[ -n "$REALM_CHK" ]]; then
  LogMess -p -w -m "Found local settings of domain."
  ExitDomain -u "$ADMIN_NAME"  -p "$ADMIN_PASS"
fi

SaveConfig

if [[ "$DOMAIN_TYPE" = "IPA" ]]; then
  JoinFreeIPA -u "$ADMIN_NAME" -n "$PC_NAME" -d "$DOMAIN_NAME" -p "$ADMIN_PASS" || JoinExit 1
  JoinExit 0
fi

GetDCNames -d "$DOMAIN_NAME"

SetAuthSelect
CreateKrb5 -d "$DOMAIN_NAME" -k "$KRB5_DC1" -r "$KRB5_DC2" -c "$DC1"
SetHostname ${PC_NAME}.$DOMAIN_NAME
TymeSync "$DOMAIN_NAME"

OS_NAME=$(cat /etc/os-release | grep ^"NAME=" | awk -F= '{print $2}' | sed 's/\"//g')
OS_VERSION=$(cat /etc/os-release | grep ^"VERSION_ID=" | awk -F= '{print $2}' | sed 's/\"//g')

if [[ -n "$WINBIND" ]]; then
  W_DOMAIN="$DOMAIN_NAME"
  JoinWinbind -d "$W_DOMAIN" -o "$OS_NAME" -v "$OS_VERSION" -u "$ADMIN_NAME" -p "$ADMIN_PASS" -c "$DC1"   || JoinExit 1
  JoinExit 0
fi

Join -d "$DOMAIN_NAME" -u "$ADMIN_NAME" -p "$ADMIN_PASS" -o "$OS_NAME" -v "$OS_VERSION" -c "$DC1" -e "$DC2" $FQN || JoinExit 1
JoinExit 0
