#!/usr/bin/bash

CURR_TIME=$(date '+%d-%m-%y %H-%M-%S')
export CURR_TIME
export LOG_FILE='/var/log/join_to_domain.log'
export VERSION='1.0.0'

BASE_DIR=$(realpath "$0")
BASE_DIR=$(dirname "$BASE_DIR")
export BASE_DIR
export PATH="${BASE_DIR}:$PATH"

USAGE_S='' DOMAIN_NAME='' PC_NAME='' ADMIN_NAME='' DC_NAME='' WG='' WINBIND='' FORCE='' DEL_PC='' LOWER_CASE='' SAVE_CASE='' DEL_SSSD_CACHE='' DNS_AUTH_NONE='' SSH_KRB5='' DOMAIN_TYPE='WIN' ADMIN_PASS='' DC1='' DC2='' KRB5_DC1='' KRB5_DC2='' OU_JOIN='' FQN='' OU='' REALM_CHK=''

VARS=$(getopt -o d:n:u:o:c:W:wDyfisrakqvh --long domain:,pc-name:,admin:,ou:,dc-name:,wg:,winbind:,delete-pc,yes,force,lower-case,save-case,del-sssd-cache,dns-auth-none,ssh-krb5,fully-qualified-names,version,help -n 'join-to-domain' -- "$@")
eval set -- "$VARS"

while true; do
  case "$1" in
    -d | --domain ) DOMAIN_NAME="$2"; shift 2 ;;
    -n | --pc-name ) PC_NAME="$2"; shift 2 ;;
    -u | --admin ) ADMIN_NAME="$2"; shift 2 ;;
    -o | --ou ) OU="$2"; shift 2 ;;
    -c | --dc-name ) DC_NAME="$2"; shift 2 ;;
    -W | --wg ) WG="$2"; shift 2 ;;
    -w | --winbind ) WINBIND="$2"; shift 2 ;;
    -D | --delete-pc ) DEL_PC='Y'; shift ;;
    -y | --yes ) AUTO_YES='Y'; export AUTO_YES; shift ;;
    -f | --force ) FORCE='Y'; shift ;;
    -i | --lower-case ) LOWER_CASE='-i'; shift ;;
    -s | --save-case ) SAVE_CASE='-s'; shift ;;
    -r | --del-sssd-cache ) DEL_SSSD_CACHE='Y'; shift ;;
    -a | --dns-auth-none ) DNS_AUTH_NONE='-a'; shift ;;
    -k | --ssh-krb5 ) SSH_KRB5='-k'; shift ;;
    -q | --fully-qualified-names ) FQN='-q'; shift ;;
    -v | --version ) echo "Version: $SCRIPT_VERSION"; exit 0 ;;
    -h | --help ) USAGE_S="Y"; shift; break ;;
    -- ) shift; break ;;
    * ) USAGE_S="Y"; break ;;
  esac
done

. "${BASE_DIR}/join-to-domain-function.sh" || exit 1
# SetColor

LogMess -S -m "Script started. Version: $SCRIPT_VERSION"
LogMess -m "Time: $CURR_TIME"

[[ -n "$USAGE_S" ]] && { Usage; exit 0; }
[[ -n "$DEL_SSSD_CACHE" ]] && { RemoveSSSDCache; exit 0; }

[[ -z "$ADMIN_NAME" ]] && { LogMess -p -e -m "Domain administrator name is not set!"; Usage; exit 1; }
[[ -z "$PC_NAME" ]] && { PC_NAME="$CURR_HOSTNAME"; }

if [[ -z "$DOMAIN_NAME" ]]; then
  if [[ -n "$CURR_DOMAIN" ]]; then
    LogMess -p -m 'Domain name is $CURR_DOMAIN'
    DOMAIN_NAME="$CURR_DOMAIN"
  else
    LogMess -p -e -m "Domain name is not set!"; Usage; exit 1;
  fi
fi

CheckPCName "$PC_NAME" || exit 1

[[ "$(id -u)" != "0" ]] && { LogMess -P -m "Run the script with root user rights."; exit 1; }
ADMIN_NAME=${ADMIN_NAME%%@*}
ADMIN_PASS=$(ReadPass)
echo

DC_OPT=''
[[ -n "$DC_NAME" ]] && DC_OPT="-c $DC_NAME"

REALM_CHK=$(realm list)

if [[ -n "$DEL_PC" ]]; then
  if CheckDomain "$DOMAIN_NAME"; then
    if ! CheckPCInDomain -u "$ADMIN_NAME" -n "$PC_NAME" -d "$DOMAIN_NAME"  -p "$ADMIN_PASS" -N; then
      if [[ -n "$REALM_CHK" ]]; then
        LogMess -p -m 'Computer account $PC_NAME is not found in domain!'
        LogMess -p -m "Remove local settings of domain?"
        ContinueAsk
        ExitDomain -u "$ADMIN_NAME"  -p "$ADMIN_PASS" -l $DC_OPT
      fi
    else
      ExitDomain -u "$ADMIN_NAME"  -p "$ADMIN_PASS" $DC_OPT
    fi
  else
    LogMess -p -e -m 'Domain $DOMAIN_NAME is not available!'
    if [[ -n "$REALM_CHK" ]]; then
      LogMess -p -m "Remove local settings of domain?"
      ContinueAsk
      ExitDomain -u "$ADMIN_NAME"  -p "$ADMIN_PASS" -l $DC_OPT
    fi
  fi
  exit 0
fi

CheckDomain "$DOMAIN_NAME"
RET_CODE=$?
case $RET_CODE in
    12) DOMAIN_TYPE="IPA" ;;
    11)  ;;
    *)   LogMess -p -e -m 'Domain $DOMAIN_NAME is not available!'
          LogMess -P -e -m  "Check your network and DNS settings."
          exit 1 ;;
esac

CheckPCInDomain -u "$ADMIN_NAME" -n "$PC_NAME" -d "$DOMAIN_NAME"  -p "$ADMIN_PASS"
RET_CODE=$?
[[ $RET_CODE -eq 11 ]] && { LogMess -p -e -m "Invalid domain administrator name or password!"; exit 1; }

if [[ $RET_CODE -eq 0 ]] ; then
  if [[ -n "$FORCE"  ]]; then
      LogMess -p -w -m 'Computer account $PC_NAME is already in the domain.'
      [[ -n "$OU" ]] && unset "$OU"
  else
      LogMess -p -e -m 'Computer account $PC_NAME is already in the domain.'
      LogMess -p -e -m "Delete this computer account in the domain or specify a different computer name."
      exit 1
  fi
fi

if [[ -n "$REALM_CHK" ]]; then
  LogMess -p -w -m "Found local settings of domain."
  ExitDomain -u "$ADMIN_NAME"  -p "$ADMIN_PASS" $DC_OPT
fi

SaveConfig

if [[ "$DOMAIN_TYPE" = "IPA" ]]; then
  JoinFreeIPA -u "$ADMIN_NAME" -n "$PC_NAME" -d "$DOMAIN_NAME" -p "$ADMIN_PASS" || JoinExit 1
  JoinExit 0
fi

if [[ -n "$OU" ]];  then
  IFS='. ' read -r -a ARR <<< "$DOMAIN_NAME"
  for ITEM in "${ARR[@]}"; do
       AS+=",DC=""$ITEM"
  done
  OU_JOIN="--computer-ou=${OU}${AS}"
  OU_JOIN="-j $OU_JOIN"
fi

if [[ -z "$DC_NAME" ]]; then
  GetDCNames -d "$DOMAIN_NAME"
else
    KRB5_DC1="kdc = $DC_NAME"
    DC1="$DC_NAME"
fi

SetAuthSelect
CreateKrb5 -d "$DOMAIN_NAME" -k "$KRB5_DC1" -r "$KRB5_DC2" -c "$DC1"
SetHostname ${PC_NAME}.$DOMAIN_NAME
TymeSync "$DOMAIN_NAME"

OS_NAME=$(cat /etc/os-release | grep ^"NAME=" | awk -F= '{print $2}' | sed 's/\"//g')
OS_VERSION=$(cat /etc/os-release | grep ^"VERSION_ID=" | awk -F= '{print $2}' | sed 's/\"//g')

if [[ -n "$WINBIND" ]]; then
  W_DOMAIN="$DOMAIN_NAME"
  WG_OPT=''
  if [[ -n "$WG" ]]; then
    W_DOMAIN="$WG"
    WG_OPT="-w $WG"
  fi
  JoinWinbind -d "$W_DOMAIN" -o "$OS_NAME" -v "$OS_VERSION" -u "$ADMIN_NAME" -p "$ADMIN_PASS" -c "$DC1" $WG_OPT $OU_JOIN || JoinExit 1
  JoinExit 0
fi

Join -d "$DOMAIN_NAME" -u "$ADMIN_NAME" -p "$ADMIN_PASS" -o "$OS_NAME" -v "$OS_VERSION" -c "$DC1" -e "$DC2" $OU_JOIN $SAVE_CASE $LOWER_CASE $DNS_AUTH_NONE $SSH_KRB5 $FQN || JoinExit 1
JoinExit 0
